library(tidyverse)
library(RSQLite)
library(ipapi)
library(leaflet)

dbConnect(SQLite(), "access.db") -> db

read_delim("access.log", delim = " ", col_names = c("ip", "null1", "user", "timestamp", "zone", "req", "status", "size", "referer", "agent", "null2")) %>%
  mutate_all(na_if, "-") %>%
  filter(user %>% is.na) %>%
  filter(!ip %in% c("192.168.0.254", "164.2.255.244")) %>%
  mutate(timestamp = timestamp %>% as.POSIXct(format = "[%d/%b/%Y:%H:%M:%S")) %>%
  separate(req, into = c("method", "url", "version"), sep = " ") %>%
  select(ip, timestamp, url, status, referer, agent) %>%
  dbWriteTable(conn = db, name = "access", value = ., append = T)

known_ips <- character(0)

db %>%
  tbl("geoip") %>%
  select(query) %>%
  collect %>%
  pull(query) -> known_ips

db %>%
  tbl("access") %>%
  select(ip) %>%
  collect %>%
  pull(ip) %>%
  unique %>%
  setdiff(known_ips) -> ips

if (length(ips) > 0)
{
  ips %>%
    geolocate %>%
    select(ip = query, city, country, lat, lon) %>%
    dbWriteTable(conn = db, name = "geoip", value = ., append = T)
}

db %>%
  tbl("access") %>%
  left_join(db %>% tbl("geoip")) %>%
  collect -> geoaccess

geoaccess %>%
  filter(status != 404,
         !agent %>% str_detect("bot")) %>%
  distinct(ip, lon, lat) %>%
  leaflet %>%
  addProviderTiles(providers$CartoDB.Positron) %>%
  addMarkers(~lon, ~lat, clusterOptions = markerClusterOptions())
